<?php
/**
 * @file
 * Contains Drupal\maestro_taskconsole\Controller\MaestroTaskConsoleController.
 */

namespace Drupal\maestro_taskconsole\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\maestro\Engine\MaestroEngine;
use Drupal\Core\Url;
use Drupal\Component\Serialization\Json;
use Drupal\maestro\Controller\MaestroOrchestrator;

class MaestroTaskConsoleController extends ControllerBase {
  
  /**
   * getTasks method
   * This method is called by the menu router for /taskconsole
   * The output of this method is the current user's task console.
   */
  public function getTasks($highlightQueueID = 0) {
    $config = \Drupal::config('maestro.settings');
    //before we do anything, let's see if we should be running the orchestrator through task console refreshes:
    if($config->get('maestro_orchestrator_task_console')) {
      $orchestrator = new MaestroOrchestrator();
      $orchestrator->orchestrate($config->get('maestro_orchestrator_token'));
    }
    $engine = new MaestroEngine();
    $engine->enableDebug();
    $engine->enableDevelopmentMode();
    
    $form = array();
    $form['task_console_table'] = array (
      '#type' => 'table',
      '#header' => array(t('Task'), t('Flow'), t('Assigned'), t('Actions')),   
      '#empty' => t('You have no tasks.')
    );

    //fetch the user's queue items
    $queueIDs = MaestroEngine::getAssignedTaskQueueIds(\Drupal::currentUser()->id());
    
    foreach($queueIDs as $queueID) {
      $highlight = '';
      $url_from_route = FALSE;
      if($highlightQueueID == $queueID){
        //set the highlight for the queue entry
        $highlight = 'maestro-highlight-task';
      }
      $queueRecord = \Drupal::entityTypeManager()->getStorage('maestro_queue')->resetCache(array($queueID));
      $queueRecord = \Drupal::entityTypeManager()->getStorage('maestro_queue')->load($queueID);
      $processID = MaestroEngine::getProcessIdFromQueueId($queueID);
      $processRecord = MaestroEngine::getProcessEntryById($processID);
      
      $form['task_console_table'][$queueID]['#attributes'] = array('class' => $highlight);
      
      $form['task_console_table'][$queueID]['task'] = array(
        '#plain_text' => $queueRecord->task_label->getString(),
      );
      
      $form['task_console_table'][$queueID]['flow'] = array(
        '#plain_text' => $processRecord->process_name->getString(),
      );
      
      $form['task_console_table'][$queueID]['assigned'] = array(
        '#plain_text' =>  \Drupal::service('date.formatter')->format($queueRecord->created->getString(), 'custom', 'Y-m-d H:i:s'),
      );
      
      /*
       * If this is an interactive Maestro task, it means we show an execute button with the link going to a pop out or executable form
       * We need to determine if we have any special handling for this interactive task 
       */
      
      try {
        $handler = $queueRecord->handler->getString();
        $handler = str_replace('@queueid', $queueID, $handler);
        $handler = str_replace('@processid', $processID, $handler);
        $x = Url::fromUserInput($handler, array('query' => array('queueid' => $queueID)));
        $url_from_route = FALSE;
      }
      catch (\InvalidArgumentException $e) {
        $url_from_route = TRUE;
      }
      $link = 'Execute';
      $templateMachineName = $engine->getTemplateIdFromProcessId($queueRecord->process_id->getString());
      $taskTemplate = $engine->getTemplateTaskByID($templateMachineName, $queueRecord->task_id->getString());
      //test to see if this is a URL that can be deduced from a Drupal route or not.
      //if it's not a route, then $url_from_route will be FALSE
      if($queueRecord->is_interactive->getString() == '1' && $url_from_route) {
        if(!array_key_exists('data', $taskTemplate)) {
          $taskTemplate['data']['modal'] = '';  //bit of a catch-all here.  just ensuring that we have a modal data element
        }
        //Let's call a hook here to let people change the name of the link to execute the task if they so choose to do so
        \Drupal::moduleHandler()->invokeAll('maestro_task_console_interactive_link_alter', array(&$link, $taskTemplate, $queueRecord));
        if(array_key_exists('modal', $taskTemplate['data']) && $taskTemplate['data']['modal'] == 'modal') {
          $redirect = isset($taskTemplate['data']['redirect_to']) ? urlencode($taskTemplate['data']['redirect_to']) : '';
          $form['task_console_table'][$queueID]['execute']['maestro_link'] =   
            array(
              '#type' => 'link',
              '#title' => $this->t($link),
              '#url' => Url::fromRoute('maestro.execute', ['queueid' => $queueID, 'modal' => 'modal']),
              '#attributes' => [
                'class' => ['use-ajax'],
                'data-dialog-type' => 'modal',
                'data-dialog-options' => Json::encode([
                    'width' => 700,
                ]),
              ],
            );
        }
        else {
          $form['task_console_table'][$queueID]['execute']['maestro_link'] = 
              array (
                '#type' => 'link',
                '#title' => $this->t($link),
                '#url' => Url::fromRoute('maestro.execute', ['queueid' => $queueID, 'modal' => 'notmodal']),
          );
        }
      }
      else {
        /*
         * This is a different type of interactive function.  What do we do?  Not sure.  Worst case scenario here, 
         * we use the handler as a URI and tack on the QueueID?
         * Should this be a callback to the task to provide a link?
         */
        //Let's call a hook here to let people change the name of the link to execute the task if they so choose to do so
        \Drupal::moduleHandler()->invokeAll('maestro_task_console_interactive_link_alter', array(&$link, $taskTemplate, $queueRecord));
        $handler = $queueRecord->handler->getString();
        //replace what we can here
        $processID = MaestroEngine::getProcessIdFromQueueId($queueID);
        $handler = str_replace('@queueid', $queueID, $handler);
        $handler = str_replace('@processid', $processID, $handler);
        if($handler != '') {
          $form['task_console_table'][$queueID]['execute']['maestro_link'] = array (
            '#type' => 'link',
            '#title' => $this->t($link),
            //'#url' => Url::fromUri($handler, array('query' => array('queueid' => $queueID))),
            '#url' => Url::fromUserInput($handler, array('query' => array('queueid' => $queueID))),
          );
        }
        else { //we don't have a valid URL here. It's actually blank.  Can't create a link to nothing.
          $form['task_console_table'][$queueID]['execute']['maestro_link'] = array (
            '#plain_text' => $this->t('Invalid Link'),
          );
        }
      }
      //provide your own execution links here if you wish
      \Drupal::moduleHandler()->invokeAll('maestro_task_console_alter_execution_link', array(&$form['task_console_table'][$queueID]['execute'], $taskTemplate, $queueRecord));
      
    }
    $form['#attached']['library'][] = 'maestro_taskconsole/maestro_taskconsole_css';
    return $form;
  }
  
  
}