<?php 

/**
 * @file
 * Definition of Drupal\maestro\Plugin\views\field\MaestroEngineActiveHandler
 */
 
namespace Drupal\maestro\Plugin\views\field;
 
use Drupal\Core\Form\FormStateInterface;
use Drupal\maestro\Engine\MaestroEngine;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Drupal\Core\Url;
use Drupal\Component\Serialization\Json;

/**
 * Field handler to create a usable link to the task via the handler field
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("maestro_active_handler")
 */
class MaestroEngineActiveHandler extends FieldPluginBase {
 
  /**
   * @{inheritdoc}
   */
  public function query() {
    // no Query to be done.
  }
 
  /**
   * Define the available options
   * @return array
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
   
 
    return $options;
  }
 
  /**
   * Provide the options form.
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    
 
    parent::buildOptionsForm($form, $form_state);
  }
 
  /**
   * @{inheritdoc}
   */
  public function render(ResultRow $values) {
    $item = $values->_entity;
    //this will ONLY work for production assignments and/or maestro queues
    if($item->getEntityTypeId() == 'maestro_production_assignments' || $item->getEntityTypeId() == 'maestro_queue') {
      //we are of the right types.  So let's get the right queue ID
      if($item->getEntityTypeId() == 'maestro_production_assignments') {
        $queueID = $item->queue_id->getString();
      }
      else {
        $queueID = $item->id->getString();
      }
      $engine = new MaestroEngine();
      $engine->enableDebug();
      $engine->enableDevelopmentMode();
      //ok, so at this point, we are going to try to pick off the active handler
      $queueRecord = MaestroEngine::getQueueEntryById($queueID);
      if($queueRecord->is_interactive->getString() == '1' && $queueRecord->task_class_name->getString() == 'MaestroInteractive') {
        $templateMachineName = $engine->getTemplateIdFromProcessId($queueRecord->process_id->getString());
        $taskTemplate = $engine->getTemplateTaskByID($templateMachineName, $queueRecord->task_id->getString());
        if(!array_key_exists('data', $taskTemplate)) {
          $taskTemplate['data']['modal'] = '';  //bit of a catch-all here.  just ensuring that we have a modal data element
        }
        
        if($taskTemplate['data']['modal'] == 'modal') {
          $redirect = isset($taskTemplate['data']['redirect_to']) ? urlencode($taskTemplate['data']['redirect_to']) : '';
          $form['task_console_table'][$queueID]['execute'] = array(
            '#type' => 'link',
            '#title' => $this->t('Execute'),
            '#url' => Url::fromRoute('maestro.execute', ['queueid' => $queueID, 'modal' => 'modal']),
            '#attributes' => [
              'class' => ['use-ajax'],
              'data-dialog-type' => 'modal',
              'data-dialog-options' => Json::encode([
                  'width' => 700,
              ]),
            ],
          );
        }
        else {
          $form['task_console_table'][$queueID]['execute'] = array (
            '#type' => 'link',
            '#title' => $this->t('Link'),
            '#url' => Url::fromRoute('maestro.execute', ['queueid' => $queueID, 'modal' => 'notmodal']),
          );
        }
      }
      else {
        /*
         * This is a different type of interactive function.  What do we do?  Not sure.  Worst case scenario here, 
         * we use the handler as a URI and tack on the QueueID?
         * Should this be a callback to the task to provide a link?
         */
        $handler = $queueRecord->handler->getString();
        //replace what we can here
        $processID = MaestroEngine::getProcessIdFromQueueId($queueID);
        $handler = str_replace('@queueid', $queueID, $handler);
        $handler = str_replace('@processid', $processID, $handler);
        if($handler != '') {
          $form['task_console_table'][$queueID]['execute'] = array (
            '#type' => 'link',
            '#title' => $this->t('Link'),
            '#url' => Url::fromUserInput($handler, array('query' => array('queueid' => $queueID))),
          );
        }
        else { //we don't have a valid URL here. It's actually blank.  Can't create a link to nothing.
          $form['task_console_table'][$queueID]['execute'] = array (
            '#plain_text' => $this->t('Invalid Link'),
          );
        }
      }
      
      return $form;
      
    }
    else {
      return '';
    }
  }
}